package org.ferris.net;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.MessageFormat;
import java.util.ResourceBundle;

/**
 * The purpose of this class is to take a URL object
 * convert it to a String, then allow the addition of
 * request query parameters (fname=michael,lname=remijan)
 * and then convert the String back to a URL object. This
 * class will use URLEncoder to make sure special characters
 * are escaped properly.
 * 
 * @author mjremij
 *
 */
public class UrlStringBuilder 
{
	private static final ResourceBundle rb = ResourceBundle.getBundle(UrlStringBuilder.class.getName());
	private static String getString(String key, Object... args) {
		String pattern = rb.getString(key);
		if (args.length > 0) {
			pattern = MessageFormat.format(pattern, args);
		}
		return pattern;
	}
	
	private StringBuilder sp;
	
	
	/*
	 * Catch and re-throw the exception
	 */
	private String encode(String value) {
		try {
			return URLEncoder.encode(value, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(getString("unsupportedEncoding"));
		}	
	}
	
	/**
	 * Default constructor
	 */
	public UrlStringBuilder() {
		this.sp = new StringBuilder();
	}
	
	/**
	 * Get <code>url.toString()</code> put it in a
	 * <code>StringBuilder</code> then append the "?"
	 * character.  After this call the <code>append(..)</code>
	 * methods to add request query parameters after the "?".
	 * If you're URL already has request query parameters in it
	 * that's ok, this class will just append more on.
	 * When done, call the <code>toUrl()</code> method
	 * to get a new URL object.
	 * 
	 * @param url
	 * @throws NetException
	 */
	public UrlStringBuilder(URL url)
	{
		this();
		String s = url.toString();	
		sp.append(s);
		if (s.indexOf('?') < 0) {
			sp.append('?');
		}
		else
		if (s.charAt(s.length() - 1) != '&') {
			sp.append('&');
		}
	}
	
	
	/**
	 * Get a new URL
	 */
	public URL toUrl() {
		try {
			return new URL(sp.toString());
		} catch (MalformedURLException e) {
			throw new RuntimeException(getString("malformedUrl", sp.toString()));
		}
	}
	
	
	/**
	 * Append name/value pair when value is a double.
	 */
	public void append(String name, double value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is an int.
	 */
	public void append(String name, int value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is an short.
	 */
	public void append(String name, short value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is a boolean.
	 */
	public void append(String name, boolean value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	/**
	 * Append name/value pair when value is a char.
	 */
	public void append(String name, char value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is a byte.
	 */
	public void append(String name, byte value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is a long.
	 */
	public void append(String name, long value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is a float.
	 */
	public void append(String name, float value) 
	{
		sp.append(name);
		sp.append("=");
		sp.append(value);
		sp.append("&");
	}
	
	
	/**
	 * Append name/value pair when value is a String,
	 * use URLEncoder to encode using encoding UTF-8
	 * to make sure special characters are handled 
	 * correctly. if value == null do not include in
	 * the query string.
	 */
	public void append(String name, String value) 
	{
		if (value != null) 
		{
			sp.append(name);
			sp.append("=");		
			sp.append(encode(value));
			sp.append("&");
		}
	}
	
	
	/**
	 * Append name/value pair when value is a Object
	 * using Object.toString() and then using 
	 * URLEncoder to encode using encoding UTF-8
	 * to make sure special characters are handled 
	 * correctly. if value == null do not include in
	 * the query string.
	 */
	public void append(String name, Object value) 
	{
		if (value != null) 
		{
			sp.append(name);
			sp.append("=");		
			sp.append(encode(value.toString()));
			sp.append("&");
		}
	}
		
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(boolean value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(char value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(byte value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(short value) {
		sp.append(value);		
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(int value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(long value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(float value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(double value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(String value) {
		sp.append(value);
	}
	
	
	/**
	 * Regular StringBuilder append
	 */
	public void append(Object value) {
		sp.append(value);
	}
	

	/**
	 * Return what's been appended to this builder
	 */
	public String toString() {
		return sp.toString();
	}
}
