package org.ferris.net.runner;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.AuthCache;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.params.CookiePolicy;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.cookie.Cookie;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.BasicAuthCache;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.log4j.Logger;
import org.ferris.net.NetException;
import org.ferris.net.NetExceptionCode;
import org.ferris.net.UrlStringBuilder;

/**
 * This class makes it easier to make a GET or POST call
 * to a URL, pass request parameters, and process the 
 * results which come back. An instance of this class can
 * be used to make multiple requests to the server and the
 * cookies for the server are automatically manged.  This is
 * especially useful if some sort of session cookie must 
 * be generated or a non-HTTP BASIC authentication login must be
 * performed before making the request.
 * 	
 * @author <a href="mailto:mjremijan@yahoo.com">Mike Remijan</a>
 */
public class HttpRequestRunner<M> 
{
    public static AuthScope proxyAuthScope;
    public static Credentials proxyCredentials;
        
    /**
     * Basic getter
     */
    private AuthScope getAuthScope() {
        return authScope;
    }

    
    /**
     * Basic setter
     */
    private void setAuthScope(AuthScope authScope) {
        this.authScope = authScope;
        log.info("Host=\"" + authScope.getHost() + "\"");
        log.info("Port=\"" + authScope.getPort() + "\"");
        log.info("Realm=\"" + authScope.getRealm() + "\"");
        log.info("Scheme=\"" + authScope.getScheme() + "\"");
    }

    
    /**
     * Basic getter
     */
    private UsernamePasswordCredentials getCredentials() {
        return credentials;
    }

    
    /**
     * Basic setter
     */
    private void setCredentials(UsernamePasswordCredentials credentials) {
        this.credentials = credentials;
        log.info("Username=" + ((credentials != null) ? "\"" + credentials.getUserName() + "\"" : "IS NULL"));
    }

    
    /**
     * Lazy initialization checking property for null and 
     * creating the new value before returning the first time.
     */
    private List<Object> getParameterValues() {
        if (parameterValues == null) {
            parameterValues = new ArrayList<Object>();
        }
        return parameterValues;
    }

    
    /**
     * Lazy initialization checking property for null and 
     * creating the new value before returning the first time.
     */
    private CookieStore getCookieStore() {
        if (cookieStore == null) {
            cookieStore = new BasicCookieStore();
        }
        return cookieStore;
    }    
    
    

    
//    private void setHttpProxyAuthentication(HttpClient httpClient)
//    {
//        //-Dhttp.proxyUser=someUserName
//        String proxyUser
//            = System.getProperty("http.proxyUser", "").trim();        
//        
//        //-Dhttp.proxyPassword=somePassword
//        String proxyPassword
//            = System.getProperty("http.proxyPassword","").trim();
//        
//        if (proxyUser.isEmpty() && proxyPassword.isEmpty()) {
//            if (log.isInfoEnabled()){log.info(String.format("No HTTP proxy credentials set - System properties http.proxyUser and http.proxyPassword are empty."));}
//        }
//        else        
//        {    
//            UsernamePasswordCredentials upc 
//                = new UsernamePasswordCredentials(proxyUser, proxyPassword);
//
//            if (log.isInfoEnabled()){log.info(String.format("Setting HTTP proxy credentials - System properties http.proxyUser=\"%s\", http.proxyPassword=\"%s\"",proxyUser, ((proxyPassword.isEmpty()) ? "<NULL>" : "*********")));}
//            httpClient.getState().setProxyCredentials(AuthScope.ANY, upc);            
//        }           
//    }
    
    
    /**
     * The "http" of the URL "http://localhost:8080/some/path/search.php".
     * By default returns "http".  Override this method
     * in sub-classes if "https" is needed instead.
     */
    protected String getScheme() {
        return "http";
    }

    /**
     * The "localhost" of the URL "http://localhost:8080/some/path/search.php".
     *  By default returns <code>getAuthScope().getHost()</code>.  Override this method
     * in sub-classes if needed.
     */
    protected String getHost() {
        return getAuthScope().getHost();
    }

    /**
     * The "8080" of the URL "http://localhost:8080/some/path/search.php".
     * By default returns <code>getAuthScope().getPort()</code> if the
     * value is &gt;= 0, otherwise returns -1.  Override this 
     * method in sub-classes if needed.
     */
    protected int getPort() {
        int p = getAuthScope().getPort();
        if (p > 0) {
            return p;
        } else {
            return -1;
        }
    }    
   
        
    private AuthScope authScope;
    private UsernamePasswordCredentials credentials;
    private List<Object> parameterValues;    
    private String cookiePolicy;
    private Logger log = Logger.getLogger(getClass());    
    private CookieStore cookieStore;
                     

    /**
     * This constructor is used if the HTTP 
     * resource is <b>NOT</b> protected by 
     * BASIC authentication.
     * 
     * @param authScope
     */
    public HttpRequestRunner(AuthScope authScope) {
        this(authScope, null);
    }

    /**
     * This constructor is used if the HTTP 
     * resource <b>IS</b> protected by 
     * BASIC authentication.
     * 
     * @param authScope
     */
    public HttpRequestRunner(AuthScope authScope, UsernamePasswordCredentials credentials) {
        setAuthScope(authScope);
        setCredentials(credentials);
    }
    
    
    /**
     * Set the cookie policy using a value from <code>CookiePolicy</code>.
     * 
     * @param cookiePolicy
     */
    public void setCookiePolicy(String cookiePolicy) {
        this.cookiePolicy = cookiePolicy;
    }

    /**
     * Get the cookie policy, if none is set the default is 
     * <code>CookiePolicy.RFC_2109</code>
     * 
     * @return
     */
    public String getCookiePolicy() {
        return (cookiePolicy == null) ? CookiePolicy.RFC_2109 : cookiePolicy;
    }  
    

    /**
     * Add the cookie to the request.
     */
    public void addCookie(String name, String content, String domain, String path) 
    {
        BasicClientCookie cookie = new BasicClientCookie(
              (name == null) ? "" : name
            , (content == null) ? "" : content
        );
        cookie.setVersion(0);
        cookie.setDomain((domain == null) ? "" : domain);
        cookie.setPath((path == null) ? "" : path);
        
        log.info("Add cookie = [" + cookie.toString() + "]");
        getCookieStore().addCookie(cookie);
    }

    
    /**
     * Add the String value of a request parameter and
     * URLEncoder is automatically used to make sure 
     * special characters are escaped properly.
     */
    public void addParameter(String parameter) {
        log.info("Add parameter value (String)=\"" + parameter + "\"");
        getParameterValues().add(parameter);
    }

    
    /**
     * Add the File value of a request parameter
     */
    public void addParameter(File parameter) {
        log.info("Add parameter value (File)=\"" + parameter + "\"");
        getParameterValues().add(parameter);
    }

    
    /**
     * @see #addParameter(String)
     */
    public void addParameters(String[] parameters) {
        if (parameters != null) {
            for (int i = 0; i < parameters.length; i++) {
                addParameter(parameters[i]);
            }
        }
    }

    
    /**
     * @see #addParameter(File)
     */
    public void addParameters(File[] parameters) {
        if (parameters != null) {
            for (int i = 0; i < parameters.length; i++) {
                addParameter(parameters[i]);
            }
        }
    }
    
    
    
    abstract class ExecutionContext
    {
        String request;
        HttpRequestParser parser;
        URL url;
        DefaultHttpClient httpClient;
        HttpHost targetHost;
        HttpRequestBase httpRequestBase;
        BasicHttpContext localContext;
        
        ExecutionContext(String request) {
            setRequest(request);
            setParser();
            setUrl();
            setTargetHost();
            setHttpClient();
            setLocalContext();            
            setHttpRequestBase();
        }        
        
        /**
         * Subclasses determin which one they should return
         */
        public abstract void setHttpRequestBase();
        
        /**
         * Basic setter.  The request is the SQL like URL a user passes to
         * this runner in order to make either a GET or a POST request to
         * a server.  A request is something like
         * {@code /context-root/path/to/resource.jsp?param1=?&param2=?&param3=?} 
         * The user uses the {@code addParameter[s](....) } methods to set values
         * for the parameters.  This class will take care of escaping any special
         * characters in the parameter values.
         * 
         * @param request {@link String} REQUIRED
         */
        private void setRequest(String request) {
            if (log.isInfoEnabled()) {
                log.info(String.format("Set Request!"));
            }
            this.request = request;
        }
        
        
        /**
         * Basic setter, relies on {@code request}
         * @see #setRequest(String)
         */
        private void setParser() {
            if (log.isInfoEnabled()) {
                log.info(String.format("Set Parser!"));
            }
            parser = new HttpRequestParser(request);
        }
        
        
        /**
         * Constructs a complete URL object which will be used to make a call. 
         * 
         * @see URLStringBuilder
         * @see #parser
         * @see #getParameterValues()
         */
        private void setUrl() 
        {
            if (log.isInfoEnabled()) {
                log.info(String.format("Set Url!"));
            }
            String resource = parser.getResource();
            List<String> parameterNames = parser.getParameters();
            
            UrlStringBuilder sp 
                    = new UrlStringBuilder();
            
            // Scheme (http || https)
            sp.append(getScheme());
            
            // This thing :)
            sp.append("://");
            
            // Host
            sp.append(getHost());
            
            // Port (if needed)
            sp.append((getPort() > 0) ? String.format(":%d", getPort()) : "");
            
            // Resource (supplied by user, parsed by parser, something like /context-root/path/to/resource.jsp)
            sp.append(resource);
            
            // Parameter names (supplied by user, parsed by parser, something like name=?&state=?&foo=?&)
            if (parameterNames != null && parameterNames.size() > 0) {
                sp.append("?");
                for (int i = 0; i < parameterNames.size(); i++) {
                    sp.append(parameterNames.get(i), getParameterValues().get(i));
                }
            }

            log.info(String.format("Url=\"%s\"",sp.toString()));
            url = sp.toUrl();
        }
        
        
        /**
         * Create an {@link HttpClient} instance and uses various resources
         * from this class to figure it (refer to the @see section below)
         * proxy configurable using {@code -Dhttp.proxyHost}, {@code -Dhttp.proxyPort}
         * 
         * @see #getCredentials()
         * @see #getAuthScope()
         * @see #getCookiePolicy() 
         */
        private void setHttpClient()
        {
            if (log.isInfoEnabled()) {
                log.info(String.format("Set HttpClient!"));
            }
            
            if (log.isInfoEnabled()){log.info("new DefaultHttpClient()");}
            httpClient = new DefaultHttpClient();

            //
            // CREDENTIALS
            //        
            if (getCredentials() != null) 
            {
                if (log.isInfoEnabled()){log.info(String.format("Set credentials for BASIC Authentication user=\"%s\", password=\"%s\"",getCredentials().getUserName(),((getCredentials().getPassword()==null) ? "NULL" : "NOT NULL")));}
                httpClient.getCredentialsProvider().setCredentials(
                     getAuthScope()
                    ,getCredentials()
                );                
            }

            //
            // COOKIE POLICY
            //
            if (log.isInfoEnabled()){log.info(String.format("Set cookie policy to %s", getCookiePolicy()));}
            httpClient.getParams().setParameter(
                ClientPNames.COOKIE_POLICY, getCookiePolicy());

            //
            // PROXY
            //
            //-Dhttp.proxyHost=proxyhostURL
            String proxyHost
                = System.getProperty("http.proxyHost", "").trim();        

            //-Dhttp.proxyPort=proxyPortNumber
            String proxyPort
                = System.getProperty("http.proxyPort","").trim();

            if (proxyHost.isEmpty() && proxyPort.isEmpty()) {
                if (log.isInfoEnabled()){log.info(String.format("No HTTP proxy host set - System properties \"http.proxyHost\" and \"http.proxyPort\" are empty."));}
            }
            else
            if (!proxyHost.isEmpty() && !proxyPort.isEmpty())
            {
                int proxyPortAsInt = -1;
                try {
                    proxyPortAsInt = Integer.parseInt(proxyPort);
                } catch (NumberFormatException e) {
                    throw new NetException(NetExceptionCode.NET00004, proxyPort);
                }
                if (log.isInfoEnabled()){log.info(String.format("Setting HTTP proxy host - System properties http.proxyHost=\"%s\", http.proxyPort=\"%d\"",proxyHost, proxyPortAsInt));}

                HttpHost proxy 
                    = new HttpHost(proxyHost, proxyPortAsInt);

                httpClient.getParams().setParameter(
                    ConnRoutePNames.DEFAULT_PROXY, proxy
                );
                
                if (proxyAuthScope != null) {
                    log.info("Credentials for proxy " + proxy.getHostName());
                    httpClient.getCredentialsProvider().setCredentials(
                        proxyAuthScope, proxyCredentials
                    );
                }                       
            } 
            else 
            if (proxyHost.isEmpty()) {
                throw new NetException(NetExceptionCode.NET00005, proxyPort);
            }
            else
            if (proxyPort.isEmpty()) {
                 throw new NetException(NetExceptionCode.NET00006, proxyHost);
            }
            
            
            //
            // REDIRECT
            //
            httpClient.setRedirectStrategy(new LaxRedirectStrategy());
        }
        

        
        /**
         * 
         */
        private void setLocalContext()
        {
            if (log.isInfoEnabled()) {
                log.info(String.format("Set LocalContext!"));
            }
            localContext = new BasicHttpContext();
            
            if (getCredentials() != null) 
            {
                if (log.isInfoEnabled()) {
                    log.info(String.format("Configure local context with pre-emptive BASIC http authentication"));
                }
                AuthCache authCache = new BasicAuthCache();
                BasicScheme basicAuth = new BasicScheme();
                authCache.put(targetHost, basicAuth);
                localContext.setAttribute(ClientContext.AUTH_CACHE, authCache);
            }
            
            if (log.isInfoEnabled()) {
                log.info(String.format("Configure local context with CookieStore"));
            }
            localContext.setAttribute(ClientContext.COOKIE_STORE, getCookieStore());   
        }
        
        
        /**
         * Set the {@code HttpHost} object representing the target host
         * of the service call.
         * 
         * @see #getHost() 
         * @see #getPort() 
         * @see #getScheme()         
         */
        private void setTargetHost() 
        {
            if (log.isInfoEnabled()) {
                log.info(String.format("Set Target Host!"));
            }
            targetHost = new HttpHost(getHost(), getPort(), getScheme());         
        }
        
        
        /**
         * Execute the call, pass the response stream to the handler.
         * 
         * @param handler
         * @return 
         */        
        List<M> execute(HttpResponseHandler<M> handler) 
        {
            try {
               
                log.info(String.format("EXECUTING request: %s", httpRequestBase.getRequestLine()));
                
                log.info(String.format("EXECUTING target: %s", targetHost));
                
                List<Cookie> cookies = getCookieStore().getCookies();
                if (cookies == null || cookies.isEmpty()) {
                    log.info(String.format("EXECUTING cookies: %s", "empty"));
                } else {
                    for (Cookie cookie : cookies) {
                        log.info(String.format("EXECUTING cookie: %s", cookie.toString()));
                    }
                } 
                
                log.info(String.format("EXECUTING go!"));
                HttpResponse response 
                    = httpClient.execute(targetHost, httpRequestBase, localContext);

                cookies = getCookieStore().getCookies();
                if (cookies == null || cookies.isEmpty()) {
                    log.info(String.format("RESPONSE cookies: %s", "empty"));
                } else {
                    for (Cookie cookie : cookies) {
                        log.info(String.format("RESPONSE cookie: %s", cookie.toString()));
                    }
                } 
                
                log.info("RESPONSE get stream to content");
                InputStream io = response.getEntity().getContent();
                
                log.info("RESPONSE pass stream to handler");
                return handler.response(io);
            }
            catch (Throwable t) {
                log.fatal(
                    String.format("Problem making request [%s] to target host [%s]",httpRequestBase.getRequestLine(), targetHost), t
                );
                throw new RuntimeException(t.getMessage(),t);
            } 
            finally {
                httpClient.getConnectionManager().shutdown();
                httpClient = null;
            }
        }
    }
    
    class ExecutionContextForGet extends ExecutionContext 
    {
        ExecutionContextForGet(String request) {
            super(request);
        } 

        /**
         * Implemented to perform an HTTP GET operation.
         * 
         * @see url
         */
        @Override
        public void setHttpRequestBase() 
        {
            try {
                httpRequestBase = new HttpGet(url.toURI());
            } catch (Throwable t) {
                 log.fatal(
                    String.format("Problem creating HttpGet object from URL \"%s\"",url.toString()), t
                );
                throw new RuntimeException(t.getMessage(),t);           
            }    
        }        
    }
    
    
    class ExecutionContextForPost extends ExecutionContext 
    {
        ExecutionContextForPost(String request) {
            super(request);
        } 

        /**
         * Implemented to perform an HTTP GET operation.
         * 
         * @see url
         */
        @Override
        public void setHttpRequestBase() 
        {
            try {
                httpRequestBase = new HttpPost(url.toURI());
                HttpPost httpPost =  (HttpPost)httpRequestBase;

                if (!getParameterValues().isEmpty())
                {
                    MultipartEntity reqEntity = new MultipartEntity();
                    String[] names = parser.getParameters().toArray(new String[0]);
                    Object[] valus = getParameterValues().toArray();

                    for (int i = 0; i < names.length; i++) 
                    {
                        String name = names[i];
                        Object valu = valus[i];

                        if (File.class.isAssignableFrom(valu.getClass())) {
                            FileBody bin = new FileBody((File) valu);
                            reqEntity.addPart(name, bin);
                        } else {
                            reqEntity.addPart(name, new StringBody(String.valueOf(valu)));
                        }
                    } 

                    httpPost.setEntity(reqEntity);
                }                
            } catch (Throwable t) {
                 log.fatal(
                    String.format("Problem creating HttpPost object from URL \"%s\"",url.toString()), t
                );
                throw new RuntimeException(t.getMessage(),t);           
            }    
        }        
    }
    
    
    /**
     * Make the HTTP request using GET
     * 
     * @param request - This value is something like:
     * /some/kind/of/path/to/resource.jsp?fname=?&lname=? It
     * must begin with a "/" character.  The query parameters
     * are normal query parameters with the "?" character
     * in place of the actual value.  This class will take
     * care of replacing and properly encoding the value
     * of the parameter before replacing the "?" character 
     * with it.
     * 
     * @param handler - What to do on each iteration of
     * the response. 
     */
    public List<M> get(String request, HttpResponseHandler<M> handler) 
    {
        return new ExecutionContextForGet(request).execute(handler);                    
    }

    /**
     * Make the HTTP request using POST
     * 
     * @param pathAndResource - This value is something like:
     * /some/kind/of/path/to/resource.jsp?fname=?&lname=? It
     * must begin with a "/" character.  The query parameters
     * are normal query parameters with the "?" character
     * in place of the actual value.  This class will take
     * care of replacing and properly encoding the value
     * of the parameter before replacing the "?" character 
     * with it. If you call addParameter() with a File 
     * object then the contents of the file will be posted
     * to the http server.
     * 
     * @param handler - What to do on each iteration of
     * the response. 
     */
    public List<M> post(String request, HttpResponseHandler<M> handler) 
    {
        return new ExecutionContextForPost(request).execute(handler); 
    }          
//
//
//
//    private AuthScope getAuthScope() {
//        return authScope;
//    }
//
//    private void setAuthScope(AuthScope authScope) {
//        this.authScope = authScope;
//        log.info("Host=\"" + authScope.getHost() + "\"");
//        log.info("Port=\"" + authScope.getPort() + "\"");
//        log.info("Realm=\"" + authScope.getRealm() + "\"");
//        log.info("Scheme=\"" + authScope.getScheme() + "\"");
//    }
//
//    private UsernamePasswordCredentials getCredentials() {
//        return credentials;
//    }
//
//    private void setCredentials(UsernamePasswordCredentials credentials) {
//        this.credentials = credentials;
//        log.info("Username=" + ((credentials != null) ? "\"" + credentials.getUserName() + "\"" : "IS NULL"));
//    }
//
//    private List<Object> getParameterValues() {
//        if (parameterValues == null) {
//            parameterValues = new ArrayList<Object>();
//        }
//        return parameterValues;
//    }
//
//    private List<Oreo> getCookieValues() {
//        if (cookieValues == null) {
//            cookieValues = new ArrayList<Oreo>();
//        }
//        return cookieValues;
//    }    
//
//    private void setHttpProxyHost(HttpClient httpClient)
//    {
//        //-Dhttp.proxyHost=proxyhostURL
//        String proxyHost
//            = System.getProperty("http.proxyHost", "").trim();        
//        
//        //-Dhttp.proxyPort=proxyPortNumber
//        String proxyPort
//            = System.getProperty("http.proxyPort","").trim();
//        
//        if (proxyHost.isEmpty() && proxyPort.isEmpty()) {
//            if (log.isInfoEnabled()){log.info(String.format("No HTTP proxy host set - System properties http.proxyHost and http.proxyPort are empty."));}
//        }
//        else
//        if (!proxyHost.isEmpty() && !proxyPort.isEmpty())
//        {
//            int proxyPortInt = -1;
//            try {
//                proxyPortInt = Integer.parseInt(proxyPort);
//            } catch (NumberFormatException e) {
//                throw new NetException(NetExceptionCode.NET00004, proxyPort);
//            }
//            if (log.isInfoEnabled()){log.info(String.format("Setting HTTP proxy host - System properties http.proxyHost=\"%s\", http.proxyPort=\"%d\"",proxyHost, proxyPortInt));}
//            httpClient.getHostConfiguration().setProxy(proxyHost, proxyPortInt);
//        } 
//        else 
//        if (proxyHost.isEmpty()) 
//        {
//            throw new NetException(NetExceptionCode.NET00005, proxyPort);
//        }
//        else
//        if (proxyPort.isEmpty()) 
//        {
//             throw new NetException(NetExceptionCode.NET00006, proxyHost);
//        }        
//        //-Dhttp.proxyUser=someUserName
//        //-Dhttp.proxyPassword=somePassword javaClassToRun
//    }
//    
//    
//    private void setHttpProxyCredentials(HttpClient httpClient)
//    {
//        //-Dhttp.proxyUser=someUserName
//        String proxyUser
//            = System.getProperty("http.proxyUser", "").trim();        
//        
//        //-Dhttp.proxyPassword=somePassword
//        String proxyPassword
//            = System.getProperty("http.proxyPassword","").trim();
//        
//        if (proxyUser.isEmpty() && proxyPassword.isEmpty()) {
//            if (log.isInfoEnabled()){log.info(String.format("No HTTP proxy credentials set - System properties http.proxyUser and http.proxyPassword are empty."));}
//        }
//        else        
//        {    
//            UsernamePasswordCredentials upc 
//                = new UsernamePasswordCredentials(proxyUser, proxyPassword);
//
//            if (log.isInfoEnabled()){log.info(String.format("Setting HTTP proxy credentials - System properties http.proxyUser=\"%s\", http.proxyPassword=\"%s\"",proxyUser, ((proxyPassword.isEmpty()) ? "<NULL>" : "*********")));}
//            httpClient.getState().setProxyCredentials(AuthScope.ANY, upc);            
//        }   
//        
//    }
//    
//    private void setHttpAuthentication(HttpClient httpClient)
//    {
//        if (getCredentials() != null) 
//        {
//            httpClient.getParams().setAuthenticationPreemptive(true);
//            httpClient.getState().setCredentials(
//                getAuthScope(), getCredentials()
//            );
//        }
//    }
//    
//    private HttpClient getHttpClient() 
//    {
//        if (httpClient == null) 
//        {
//            httpClient = new HttpClient();        
//            setHttpAuthentication(httpClient);
//            setHttpProxyHost(httpClient);
//            setHttpProxyCredentials(httpClient);            
//        }
//        
//        return httpClient;
//    }
//
//    /**
//     * Constructs a base URL object for a service which looks
//     * something like "http://localhost:8080/some/path/search.php".
//     * Service implementations can then use the <code>getUrl()</code> 
//     * method to create URL object with query parameters so
//     * data can be passed to the service.  
//     * 
//     * @see #getUrl()
//     * @see URLStringBuilder
//     */
//    private URL getUrl(String resource, List<String> parameterNames) {
//        UrlStringBuilder sp = new UrlStringBuilder();
//        sp.append(getScheme());
//        sp.append("://");
//        sp.append(getHost());
//        if (getPort().length() > 0) {
//            sp.append(":");
//            sp.append(getPort());
//        }
//        sp.append(resource);
//
//        if (parameterNames != null && parameterNames.size() > 0) {
//            sp.append("?");
//            for (int i = 0; i < parameterNames.size(); i++) {
//                sp.append(
//                        parameterNames.get(i), getParameterValues().get(i));
//            }
//        }
//
//        log.info("Url=\"" + sp.toString() + "\"");
//        return sp.toUrl();
//    }
//
//    private HttpMethodBase getGetMethod(URL url) {
//        GetMethod get = new MyGetMethod(url.toString());
//        return get;
//    }
//
//    class MyGetMethod extends GetMethod {
//
//        public MyGetMethod(String s) {
//            super(s);
//            setDoAuthentication(true);
//            getParams().setCookiePolicy(getCookiePolicy());
//        }
//
//        public boolean getFollowRedirects() {
//            return true;
//        }
//
//        public void setFollowRedirects(boolean b) {
//        }
//    }
//
//    class MyPostMethod extends PostMethod {
//
//        public MyPostMethod(String s) {
//            super(s);
//            setDoAuthentication(true);
//            getParams().setCookiePolicy(getCookiePolicy());
//        }
//
//        public boolean getFollowRedirects() {
//            return true;
//        }
//
//        public void setFollowRedirects(boolean b) {
//        }
//    }
//
//    private HttpMethodBase getPostMethod(URL url, List<String> parameterNames) {
//        PostMethod post = new MyPostMethod(url.toString());
//        post.getParams().setBooleanParameter(HttpMethodParams.USE_EXPECT_CONTINUE, true);
//
//        if (parameterNames != null && parameterNames.size() > 0) {
//            Part[] parts = new Part[parameterNames.size()];
//            for (int i = 0; i < parameterNames.size(); i++) {
//                String name = parameterNames.get(i);
//                Object valu = getParameterValues().get(i);
//
//                if (File.class.isAssignableFrom(valu.getClass())) {
//                    try {
//                        parts[i] = new FilePart(name, (File) valu);
//                    } catch (FileNotFoundException e) {
//                        throw new NetException(
//                                NetExceptionCode.NET00003, e, valu);
//                    }
//                } else {
//                    parts[i] = new StringPart(name, valu.toString());
//                }
//            }
//
//            post.setRequestEntity(
//                    new MultipartRequestEntity(parts, post.getParams()));
//        }
//
//        return post;
//    }
//
//    private List<M> executeMethod(HttpClient client, HttpMethodBase base, HttpResponseHandler<M> handler) {
//        try {
//            log.info("Making request");
//            int status = client.executeMethod(base);
//            if (status == HttpStatus.SC_OK) {
//                log.info("Status OK");
//
//                log.info("Cookies Received:");
//                for (Cookie c : client.getState().getCookies()) {
//                    log.info(c);
//                }
//
//                log.info("Get stream to response");
//                List<M> retval = handler.response(base.getResponseBodyAsStream());
//
//                log.info("Return list of objects");
//                return retval;
//            } else {
//                throw new NetException(
//                        NetExceptionCode.NET00001, getUri(base), status, HttpStatus.getStatusText(status));
//            }
//        } catch (NetException e) {
//            throw e;
//        } catch (Throwable t) {
//            throw new NetException(
//                    NetExceptionCode.NET00002, t, getUri(base));
//        } finally {
//            try {
//                base.releaseConnection();
//            } catch (Throwable ignore) {
//            }
//        }
//    }
//
//    private URI getUri(HttpMethodBase base) {
//        try {
//            return base.getURI();
//        } catch (Exception e) {
//            throw new RuntimeException(e);
//        }
//    }
//
//    private void setCookies(HttpMethodBase base) {
//        for (Oreo oreo : getCookieValues()) {
//            log.info("Adding cookie to request header: " + oreo.toString());
//            base.addRequestHeader("Cookie", oreo.toString());
//        }
//    }
//    
//    
//    
//    private AuthScope authScope;
//    private UsernamePasswordCredentials credentials;
//    private List<Object> parameterValues;
//    private List<Oreo> cookieValues;
//    private String cookiePolicy;
//    private Logger log = Logger.getLogger(getClass());    
//    private HttpClient httpClient;
//    
//    
//    
//    /**
//     * Set the cookie policy using a value from <code>CookiePolicy</code>.
//     * 
//     * @param cookiePolicy
//     */
//    public void setCookiePolicy(String cookiePolicy) {
//        this.cookiePolicy = cookiePolicy;
//    }
//
//    /**
//     * Get the cookie policy, if none is set the default is 
//     * <code>CookiePolicy.RFC_2109</code>
//     * 
//     * @return
//     */
//    public String getCookiePolicy() {
//        return (cookiePolicy == null) ? CookiePolicy.RFC_2109 : cookiePolicy;
//    }
//
//    private class Oreo {
//
//        public String name, content, domain, path;
//
//        public String toString() {
//            return name + "=" + content
//                    + "; "
//                    + "Domain=" + domain
//                    + "; "
//                    + "Path=" + path;
//        }
//    }
//
//    /**
//     * The "http" of the URL "http://localhost:8080/some/path/search.php".
//     * By default returns "http".  Override this method
//     * in sub-classes if "https" is needed instead.
//     */
//    protected String getScheme() {
//        return "http";
//    }
//
//    /**
//     * The "localhost" of the URL "http://localhost:8080/some/path/search.php".
//     *  By default returns <code>getAuthScope().getHost()</code>.  Override this method
//     * in sub-classes if needed.
//     */
//    protected String getHost() {
//        return getAuthScope().getHost();
//    }
//
//    /**
//     * The "8080" of the URL "http://localhost:8080/some/path/search.php".
//     * By default returns <code>getAuthScope().getPort()</code> if the
//     * value is &gt;= 0, otherwise returns "".  Override this method
//     * in sub-classes if needed.
//     */
//    protected String getPort() {
//        int p = getAuthScope().getPort();
//        if (p > 0) {
//            return String.valueOf(p);
//        } else {
//            return "";
//        }
//    }
//
//    /**
//     * This constructor is used if the HTTP 
//     * resource is <b>NOT</b> protected by 
//     * BASIC authentication.
//     * 
//     * @param authScope
//     */
//    public HttpRequestRunner(AuthScope authScope) {
//        this(authScope, null);
//    }
//
//    /**
//     * This constructor is used if the HTTP 
//     * resource <b>IS</b> protected by 
//     * BASIC authentication.
//     * 
//     * @param authScope
//     */
//    public HttpRequestRunner(AuthScope authScope, UsernamePasswordCredentials credentials) {
//        setAuthScope(authScope);
//        setCredentials(credentials);
//    }
//
//    /**
//     * Add the cookie to the request.
//     */
//    public void addCookie(String name, String content, String domain, String path) {
//        Oreo o = new Oreo();
//        o.name = (name == null) ? "" : name;
//        o.content = (content == null) ? "" : content;
//        o.domain = (domain == null) ? "" : domain;
//        o.path = (path == null) ? "" : path;
//
//        log.info("Add cookie value = [" + o.toString() + "]");
//        getCookieValues().add(o);
//    }
//
//    /**
//     * Add the String value of a request parameter and
//     * URLEncoder is automatically used to make sure 
//     * special characters are escaped properly.
//     */
//    public void addParameter(String parameter) {
//        log.info("Add parameter value (String)=\"" + parameter + "\"");
//        getParameterValues().add(parameter);
//    }
//
//    /**
//     * Add the File value of a request parameter
//     */
//    public void addParameter(File parameter) {
//        log.info("Add parameter value (File)=\"" + parameter + "\"");
//        getParameterValues().add(parameter);
//    }
//
//    /**
//     * @see #addParameter(String)
//     */
//    public void addParameters(String[] parameters) {
//        if (parameters != null) {
//            for (int i = 0; i < parameters.length; i++) {
//                addParameter(parameters[i]);
//            }
//        }
//    }
//
//    /**
//     * @see #addParameter(File)
//     */
//    public void addParameters(File[] parameters) {
//        if (parameters != null) {
//            for (int i = 0; i < parameters.length; i++) {
//                addParameter(parameters[i]);
//            }
//        }
//    }
//
//    /**
//     * Make the HTTP request using GET
//     * 
//     * @param request - This value is something like:
//     * /some/kind/of/path/to/resource.jsp?fname=?&lname=? It
//     * must begin with a "/" character.  The query parameters
//     * are normal query parameters with the "?" character
//     * in place of the actual value.  This class will take
//     * care of replacing and properly encoding the value
//     * of the parameter before replacing the "?" character 
//     * with it.
//     * 
//     * @param handler - What to do on each iteration of
//     * the response. 
//     */
//    public List<M> get(String request, HttpResponseHandler<M> handler) {
//        HttpRequestParser p = new HttpRequestParser(request);
//        URL url = getUrl(p.getResource(), p.getParameters());
//        HttpClient client = getHttpClient();
//        HttpMethodBase get = getGetMethod(url);
//        setCookies(get);
//
//        return executeMethod(client, get, handler);
//    }
//
//    /**
//     * Make the HTTP request using POST
//     * 
//     * @param pathAndResource - This value is something like:
//     * /some/kind/of/path/to/resource.jsp?fname=?&lname=? It
//     * must begin with a "/" character.  The query parameters
//     * are normal query parameters with the "?" character
//     * in place of the actual value.  This class will take
//     * care of replacing and properly encoding the value
//     * of the parameter before replacing the "?" character 
//     * with it. If you call addParameter() with a File 
//     * object then the contents of the file will be posted
//     * to the http server.
//     * 
//     * @param handler - What to do on each iteration of
//     * the response. 
//     */
//    public List<M> post(String request, HttpResponseHandler<M> handler) {
//        HttpRequestParser p = new HttpRequestParser(request);
//        URL url = getUrl(p.getResource(), null);
//        HttpClient client = getHttpClient();
//        HttpMethodBase post = getPostMethod(url, p.getParameters());
//        setCookies(post);
//
//        return executeMethod(client, post, handler);
//    }
}
